#The OMC-2900 format is enabled via the config.txt file:
#
#   # ----Output-settings---- #
#   OMC2900:
#   - id: output0
#     port: serial4
#     mode: RS232
#     baudrate: 9600
#     sensor_data_print: False
#
#  The other port settings are defined identically as is done with the sensor drivers. 
#  Also multiple ports can be defined by adding multiple id's, as is done with the sensor drivers
#
#  If 'sensor_data_print' is set to 'True' the USB REPL output will print all obtained sensor data.
#  This data is used to identify the required sensor data for omc2900 output.
#  'sensor_data_print' is found under:
#
#   # ----System---- #
#   Omc048:
#     system_id: Test
#     application: Test
#     file_log_level: info
#     repl_log_level: info
#     utc_time_offset_hours: +1
#     utc_time_offset_minutes: 0
#     sensor_data_print: True
#
# Output over usb example :
#
#     Sensor data: ('Temp C', 'C', 'TMP_600OMS') 20.11
#     Sensor data: ('SpCond ms/cm', 'mS/cm', 'SPC_600OMS') 0.00
#     Sensor data: ('ODOsat %', '%', 'ODO_600OMS') 92.5
#     Sensor data: ('ODO mg/L', 'mg/L', 'ODO_600OMS') 8.39
#
# The parameter fields can be copied to the desired OMC2900 conversion in the 'self.conversion_dict' below
# The dictionary can be tailored if needed.
#
# Once sensor data is received, the parameters are ran through the dictionary and all matched parameters 
# shall automatically transmit the omc-2900 message (via the serial_output_driver) over the configured output port.

import obs
from sensor_drv import SensorDrv
        
__syslog = obs.system_log('OMC2900')

class OMC2900(SensorDrv):

    def __init__(self, serial, conf):
        SensorDrv.__init__(self)
        self.conversion_dict =  {
                                    ('Coin cell voltage', 'V', 'VBAT')          :   "a",   #Battery voltage logger      "V"
                                    ('Temperature', 'C', 'TEMP')                :   "b",   #Temperature cabinet         "C"
                                    ('Humidity', 'rh', 'HUM')                   :   "c",   #Humidity cabinet            "%"
                                    #None                                        :   "d",   #Battery capacity            "mAh"
                                    #None                                        :   "e",   #Status                      ""
                                    #None                                        :   "h",   #GPS LAT                     "degree"
                                    #None                                        :   "i",   #GPS LON                     "degree"
                                    #None                                        :   "j",   #GPS signal quality          "%"
                                    #None                                        :   "l",   #Cellular Status             ""
                                    #None                                        :   "m",   #Cellular signal quality     "%"
                                    #None                                        :   "o",   #Barometric pressure         "hPa"
                                    #None                                        :   "p",   #Outside air temperature     "C"
                                    #None                                        :   "q",   #Outside humidity            "%"
                                    #None                                        :   "s",   #Absolute pressure           "hPa"
                                    #None                                        :   "t",   #Relative pressure           "hPa"
                                    #None                                        :   "u",   #Water column above sensor   "m"
                                    #None                                        :   "v",   #Water level NAP             "m"

                                    #None                                        :   "A",   #Temperature                 "C"
                                    #None                                        :   "B",   #Turbidity                   "NTU/FNU"
                                    #None                                        :   "C",   #Conductivity                "mS/cm" 
                                    #None                                        :   "D",   #Specific conductivity       "mS/cm"
                                    #None                                        :   "E",   #Salinity                    "PSU"
                                    #None                                        :   "F",   #DO                          "mg/l"
                                    #None                                        :   "G",   #DO                          "%"
                                    #None                                        :   "H",   #pH                          "pH"
                                    #None                                        :   "I",   #ORP                         "mV"
                                    #None                                        :   "J",   #BGA                         "µg/l"
                                    #None                                        :   "K",   #BGA                         "RFU"
                                    #None                                        :   "L",   #Chlorophylle                "µg/l"
                                    #None                                        :   "M",   #Chlorophylle                "RFU"
                                    #None                                        :   "N",   #Nitrate (NO3-N)             "mg/l"
                                    #None                                        :   "O",   #Chloride                    "mg/l"
                                    #None                                        :   "P",   #Ammonia                     "mg/l"
                                    #None                                        :   "Q",   #Orthophosphate (PO4-P)      "mg/l"
                                    #None                                        :   "R",   #Nitrogen (total)            "mg/l"
                                    #None                                        :   "S",   #Phosphorous (total)         "mg/l"
                                    #None                                        :   "V",   #PAR 1                       "microEinsteins / (m2 sec)"
                                    #None                                        :   "W",   #PAR 2                       "microEinsteins / (m2 sec)"
                                    #None                                        :   "X",   #PAR 3                       "microEinsteins / (m2 sec)"
                                }

    def convert_para_data_message(self, parameter, data):
        for param in self.conversion_dict:
            if param == parameter:
                message = '\n{}{} \r'.format(self.conversion_dict[param], data)
                message = '\n{}{} {:02X}\r'.format(self.conversion_dict[param], data, self.checksum(message))
                return message

    def checksum(self, omc2900_string):
        it = iter(map(ord, omc2900_string))
        value = next(it)
        for element in it:
            value += element
        value = value % 0x100
        return value
